@@ -225,7 +225,7 @@ static NEOERR *_add_cgi_env_var (CGI *cgi, char *env, char *name)
   if (s != NULL)
   {
     err = hdf_set_buf (cgi->hdf, name, s);
-    if (err != STATUS_OK) 
+    if (err != STATUS_OK)
     {
       free(s);
       return nerr_pass (err);
@@ -234,11 +234,12 @@ static NEOERR *_add_cgi_env_var (CGI *cgi, char *env, char *name)
   return STATUS_OK;
 }
 
-unsigned char *cgi_url_unescape (unsigned char *s)
+char *cgi_url_unescape (char *value)
 {
   int i = 0, o = 0;
+  unsigned char *s = (unsigned char *)value;
 
-  if (s == NULL) return s;
+  if (s == NULL) return value;
   while (s[i])
   {
     if (s[i] == '+')
@@ -260,13 +261,14 @@ unsigned char *cgi_url_unescape (unsigned char *s)
     }
   }
   if (i && o) s[o] = '\0';
-  return s;
+  return (char *)s;
 }
 
-NEOERR *cgi_js_escape (unsigned char *buf, unsigned char **esc)
+NEOERR *cgi_js_escape (const char *in, char **esc)
 {
   int nl = 0;
   int l = 0;
+  unsigned char *buf = (unsigned char *)in;
   unsigned char *s;
 
   while (buf[l])
@@ -275,14 +277,14 @@ NEOERR *cgi_js_escape (unsigned char *buf, unsigned char **esc)
 	buf[l] == '\\' || buf[l] == '>' || buf[l] == '<' || buf[l] < 32)
     {
       nl += 3;
-    } 
+    }
     nl++;
     l++;
   }
 
-  s = (char *) malloc (sizeof(char) * (nl + 1));
-  if (s == NULL) 
-    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to escape %s", 
+  s = (unsigned char *) malloc (sizeof(unsigned char) * (nl + 1));
+  if (s == NULL)
+    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to escape %s",
 	buf);
 
   nl = 0; l = 0;
@@ -304,33 +306,55 @@ NEOERR *cgi_js_escape (unsigned char *buf, unsigned char **esc)
   }
   s[nl] = '\0';
 
-  *esc = s;
+  *esc = (char *)s;
   return STATUS_OK;
 }
 
-NEOERR *cgi_url_escape_more (unsigned char *buf, unsigned char **esc, unsigned char *other)
+// List of all characters that must be escaped
+// List based on http://www.blooberry.com/indexdot/html/topics/urlencoding.htm
+static char EscapedChars[] = "$&+,/:;=?@ \"<>#%{}|\\^~[]`";
+
+// Check if a single character needs to be escaped
+static BOOL is_reserved_char(char c)
+{
+  int i = 0;
+
+  if (c < 32 || c > 122) {
+    return TRUE;
+  } else {
+    while (EscapedChars[i]) {
+      if (c == EscapedChars[i]) {
+        return TRUE;
+      }
+      ++i;
+    }
+  }
+  return FALSE;
+}
+
+NEOERR *cgi_url_escape_more (const char *in, char **esc,
+                             const char *other)
 {
   int nl = 0;
   int l = 0;
   int x = 0;
+  unsigned char *buf = (unsigned char *)in;
+  unsigned char *uother = (unsigned char *)other;
   unsigned char *s;
   int match = 0;
 
   while (buf[l])
   {
-    if (buf[l] == '/' || buf[l] == '+' || buf[l] == '=' || buf[l] == '&' || 
-	buf[l] == '"' || buf[l] == '%' || buf[l] == '?' || buf[l] == '#' ||
-	buf[l] == '<' || buf[l] == '>' || buf[l] == '\'' ||
-        buf[l] < 32 || buf[l] > 122)
+    if (is_reserved_char(buf[l]))
     {
       nl += 2;
-    } 
-    else if (other)
+    }
+    else if (uother)
     {
       x = 0;
-      while (other[x])
+      while (uother[x])
       {
-	if (other[x] == buf[l])
+	if (uother[x] == buf[l])
 	{
 	  nl +=2;
 	  break;
@@ -343,8 +367,8 @@ NEOERR *cgi_url_escape_more (unsigned char *buf, unsigned char **esc, unsigned c
   }
 
   s = (unsigned char *) malloc (sizeof(unsigned char) * (nl + 1));
-  if (s == NULL) 
-    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to escape %s", 
+  if (s == NULL)
+    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to escape %s",
 	buf);
 
   nl = 0; l = 0;
@@ -358,19 +382,16 @@ NEOERR *cgi_url_escape_more (unsigned char *buf, unsigned char **esc, unsigned c
     }
     else
     {
-      if (buf[l] == '/' || buf[l] == '+' || buf[l] == '=' || buf[l] == '&' || 
-	  buf[l] == '"' || buf[l] == '%' || buf[l] == '?' || buf[l] == '#' ||
-	  buf[l] == '<' || buf[l] == '>' || buf[l] == '\'' ||
-          buf[l] < 32 || buf[l] > 122)
+      if (is_reserved_char(buf[l]))
       {
 	match = 1;
       }
-      else if (other)
+      else if (uother)
       {
 	x = 0;
-	while (other[x])
+	while (uother[x])
 	{
-	  if (other[x] == buf[l])
+	  if (uother[x] == buf[l])
 	  {
 	    match = 1;
 	    break;
@@ -393,11 +414,11 @@ NEOERR *cgi_url_escape_more (unsigned char *buf, unsigned char **esc, unsigned c
   }
   s[nl] = '\0';
 
-  *esc = s;
+  *esc = (char *)s;
   return STATUS_OK;
 }
 
-NEOERR *cgi_url_escape (unsigned char *buf, unsigned char **esc)
+NEOERR *cgi_url_escape (const char *buf, char **esc)
 {
   return nerr_pass(cgi_url_escape_more(buf, esc, NULL));
 }
@@ -429,7 +450,7 @@ static NEOERR *_parse_query (CGI *cgi, char *query)
 
 
       /* Check for some invalid query strings */
-      if (*k == 0) { 
+      if (*k == 0) {
         /*  '?=foo' gets mapped in as Query._1=foo */
         snprintf(unnamed,sizeof(unnamed), "_%d", unnamed_count++);
         k = unnamed;
@@ -504,11 +525,11 @@ static NEOERR *_parse_post_form (CGI *cgi)
   cgi->data_expected = len;
 
   query = (char *) malloc (sizeof(char) * (len + 1));
-  if (query == NULL) 
-    return nerr_raise (NERR_NOMEM, 
+  if (query == NULL)
+    return nerr_raise (NERR_NOMEM,
 	"Unable to allocate memory to read POST input of length %d", l);
 
-  
+
   o = 0;
   while (o < len)
   {
@@ -538,7 +559,7 @@ static NEOERR *_parse_cookie (CGI *cgi)
 {
   NEOERR *err;
   char *cookie;
-  char *k, *v, *l; 
+  char *k, *v, *l;
   HDF *obj;
 
   err = hdf_get_copy (cgi->hdf, "HTTP.Cookie", &cookie, NULL);
@@ -546,7 +567,7 @@ static NEOERR *_parse_cookie (CGI *cgi)
   if (cookie == NULL) return STATUS_OK;
 
   err = hdf_set_value (cgi->hdf, "Cookie", cookie);
-  if (err != STATUS_OK) 
+  if (err != STATUS_OK)
   {
     free(cookie);
     return nerr_pass(err);
@@ -563,12 +584,12 @@ static NEOERR *_parse_cookie (CGI *cgi)
       v = l;
       while (*l && *l != ';') l++;
       if (*l) *l++ = '\0';
-    } 
+    }
     else
     {
       v = "";
       if (*l) *l++ = '\0';
-    } 
+    }
     k = neos_strip (k);
     v = neos_strip (v);
     if (k[0] && v[0])
@@ -695,8 +716,8 @@ static NEOERR *cgi_pre_parse (CGI *cgi)
   return STATUS_OK;
 }
 
-NEOERR *cgi_register_parse_cb(CGI *cgi, char *method, char *ctype, void *rock, 
-    CGI_PARSE_CB parse_cb)
+NEOERR *cgi_register_parse_cb(CGI *cgi, const char *method, const char *ctype,
+                              void *rock, CGI_PARSE_CB parse_cb)
 {
   struct _cgi_parse_cb *my_pcb;
 
@@ -713,7 +734,7 @@ NEOERR *cgi_register_parse_cb(CGI *cgi, char *method, char *ctype, void *rock,
   {
     free(my_pcb);
     return nerr_raise(NERR_NOMEM, "Unable to allocate memory to register parse cb");
-  } 
+  }
   if (!strcmp(my_pcb->method, "*"))
     my_pcb->any_method = 1;
   if (!strcmp(my_pcb->ctype, "*"))
@@ -833,7 +854,7 @@ NEOERR *cgi_parse (CGI *cgi)
     if (!unlink_files)
     {
       char *name;
-      err = uListGet(cgi->filenames, uListLength(cgi->filenames)-1, 
+      err = uListGet(cgi->filenames, uListLength(cgi->filenames)-1,
 	  (void *)&name);
       if (err) return nerr_pass(err);
       err = hdf_set_value (cgi->hdf, "PUT.FileName", name);
@@ -870,7 +891,7 @@ NEOERR *cgi_init (CGI **cgi, HDF *hdf)
 
   mycgi->ignore_empty_form_vars = IgnoreEmptyFormVars;
 
-  do 
+  do
   {
     if (hdf == NULL)
     {
@@ -922,7 +943,7 @@ void cgi_destroy (CGI **cgi)
 
 static NEOERR *render_cb (void *ctx, char *buf)
 {
-  STRING *str= (STRING *)ctx;
+  STRING *str = (STRING *)ctx;
   NEOERR *err;
 
   err = nerr_pass(string_append(str, buf));
@@ -941,7 +962,7 @@ static NEOERR *cgi_headers (CGI *cgi)
     /* this isn't in any HTTP rfc's, it just seems to be a convention */
     err = cgiwrap_writef ("Pragma: no-cache\r\n");
     if (err != STATUS_OK) return nerr_pass (err);
-    err = cgiwrap_writef ("Expires: Fri, 01 Jan 1990 00:00:00 GMT\r\n"); 
+    err = cgiwrap_writef ("Expires: Fri, 01 Jan 1990 00:00:00 GMT\r\n");
     if (err != STATUS_OK) return nerr_pass (err);
     err = cgiwrap_writef ("Cache-control: no-cache, must-revalidate, no-cache=\"Set-Cookie\", private\r\n");
     if (err != STATUS_OK) return nerr_pass (err);
@@ -998,9 +1019,9 @@ static NEOERR *cgi_compress (STRING *str, char *obuf, int *olen)
 
   stream.next_in = (Bytef*)str->buf;
   stream.avail_in = (uInt)str->len;
-  stream.next_out = obuf;
+  stream.next_out = (Bytef*)obuf;
   stream.avail_out = (uInt)*olen;
-  if ((uLong)stream.avail_out != *olen) 
+  if ((uLong)stream.avail_out != *olen)
     return nerr_raise(NERR_NOMEM, "Destination too big: %ld", *olen);
 
   stream.zalloc = (alloc_func)0;
@@ -1009,7 +1030,7 @@ static NEOERR *cgi_compress (STRING *str, char *obuf, int *olen)
 
   /* err = deflateInit(&stream, Z_DEFAULT_COMPRESSION); */
   err = deflateInit2(&stream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -MAX_WBITS, DEF_MEM_LEVEL, Z_DEFAULT_STRATEGY);
-  if (err != Z_OK) 
+  if (err != Z_OK)
     return nerr_raise(NERR_SYSTEM, "deflateInit2 returned %d", err);
 
   err = deflate(&stream, Z_FINISH);
@@ -1049,14 +1070,14 @@ static void debug_output(char *header, char *s, int n)
   }
   fprintf(stderr, "\n");
 }
-#endif 
+#endif
 
 void cgi_html_ws_strip(STRING *str, int level)
 {
   int ws = 0;
   int seen_nonws = level > 1;
   int i, o, l;
-  unsigned char *ch;
+  char *ch;
 
   i = o = 0;
   if (str->len) {
@@ -1146,7 +1167,7 @@ void cgi_html_ws_strip(STRING *str, int level)
       {
 	i++;
       }
-      else 
+      else
       {
 	str->buf[o++] = str->buf[i++];
 	ws = 1;
@@ -1239,12 +1260,12 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
     }
     if (use_deflate)
     {
-      err = hdf_set_value (cgi->hdf, "cgiout.other.encoding", 
+      err = hdf_set_value (cgi->hdf, "cgiout.other.encoding",
 	  "Content-Encoding: deflate");
     }
     else if (use_gzip)
     {
-      err = hdf_set_value (cgi->hdf, "cgiout.other.encoding", 
+      err = hdf_set_value (cgi->hdf, "cgiout.other.encoding",
 	  "Content-Encoding: gzip");
     }
     if (err != STATUS_OK) return nerr_pass(err);
@@ -1285,7 +1306,7 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
 	if (k == NULL) break;
 	err =string_appendf (str, "%s = %s<br>", k, v);
 	if (err != STATUS_OK) return nerr_pass(err);
-	free(k);  
+	free(k);
 	free(v);
 	x++;
       }
@@ -1308,7 +1329,7 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
       if (use_gzip)
       {
 	crc = crc32(0L, Z_NULL, 0);
-	crc = crc32(crc, str->buf, str->len);
+	crc = crc32(crc, (const Bytef *)(str->buf), str->len);
       }
       len2 = str->len * 2;
       dest = (char *) malloc (sizeof(char) * len2);
@@ -1325,7 +1346,7 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
 	       * embedded NULLs... though I should fix the python one
 	       * now as well */
 	      sprintf(gz_buf, "%c%c%c%c%c%c%c%c%c%c", gz_magic[0], gz_magic[1],
-		  Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/, 
+		  Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/,
 		  OS_CODE);
 	      err = cgiwrap_write(gz_buf, 10);
 	    }
@@ -1336,14 +1357,14 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
 	    if (use_gzip)
 	    {
 	      /* write crc and len in network order */
-	      sprintf(gz_buf, "%c%c%c%c%c%c%c%c", 
-		  (0xff & (crc >> 0)), 
-		  (0xff & (crc >> 8)), 
-		  (0xff & (crc >> 16)), 
+	      sprintf(gz_buf, "%c%c%c%c%c%c%c%c",
+		  (0xff & (crc >> 0)),
+		  (0xff & (crc >> 8)),
+		  (0xff & (crc >> 16)),
 		  (0xff & (crc >> 24)),
-		  (0xff & (str->len >> 0)), 
-		  (0xff & (str->len >> 8)), 
-		  (0xff & (str->len >> 16)), 
+		  (0xff & (str->len >> 0)),
+		  (0xff & (str->len >> 8)),
+		  (0xff & (str->len >> 16)),
 		  (0xff & (str->len >> 24)));
 	      err = cgiwrap_write(gz_buf, 8);
 	      if (err != STATUS_OK) break;
@@ -1371,17 +1392,17 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
   return nerr_pass(err);
 }
 
-NEOERR *cgi_html_escape_strfunc(unsigned char *str, unsigned char **ret)
+NEOERR *cgi_html_escape_strfunc(const char *str, char **ret)
 {
   return nerr_pass(html_escape_alloc(str, strlen(str), ret));
 }
 
-NEOERR *cgi_html_strip_strfunc(unsigned char *str, unsigned char **ret)
+NEOERR *cgi_html_strip_strfunc(const char *str, char **ret)
 {
   return nerr_pass(html_strip_alloc(str, strlen(str), ret));
 }
 
-NEOERR *cgi_text_html_strfunc(unsigned char *str, unsigned char **ret)
+NEOERR *cgi_text_html_strfunc(const char *str, char **ret)
 {
   return nerr_pass(convert_text_html_alloc(str, strlen(str), ret));
 }
@@ -1421,7 +1442,7 @@ NEOERR *cgi_cs_init(CGI *cgi, CSPARSE **cs)
   return nerr_pass(err);
 }
 
-NEOERR *cgi_display (CGI *cgi, char *cs_file)
+NEOERR *cgi_display (CGI *cgi, const char *cs_file)
 {
   NEOERR *err = STATUS_OK;
   char *debug;
@@ -1480,7 +1501,7 @@ void cgi_neo_error (CGI *cgi, NEOERR *err)
   cgiwrap_writef("</pre></body></html>\n");
 }
 
-void cgi_error (CGI *cgi, char *fmt, ...)
+void cgi_error (CGI *cgi, const char *fmt, ...)
 {
   va_list ap;
 
@@ -1522,7 +1543,7 @@ void cgi_debug_init (int argc, char **argv)
   }
 }
 
-void cgi_vredirect (CGI *cgi, int uri, char *fmt, va_list ap)
+void cgi_vredirect (CGI *cgi, int uri, const char *fmt, va_list ap)
 {
   cgiwrap_writef ("Status: 302\r\n");
   cgiwrap_writef ("Content-Type: text/html\r\n");
@@ -1575,7 +1596,7 @@ void cgi_vredirect (CGI *cgi, int uri, char *fmt, va_list ap)
 
 }
 
-void cgi_redirect (CGI *cgi, char *fmt, ...)
+void cgi_redirect (CGI *cgi, const char *fmt, ...)
 {
   va_list ap;
 
@@ -1585,7 +1606,7 @@ void cgi_redirect (CGI *cgi, char *fmt, ...)
   return;
 }
 
-void cgi_redirect_uri (CGI *cgi, char *fmt, ...)
+void cgi_redirect_uri (CGI *cgi, const char *fmt, ...)
 {
   va_list ap;
 
@@ -1595,7 +1616,7 @@ void cgi_redirect_uri (CGI *cgi, char *fmt, ...)
   return;
 }
 
-char *cgi_cookie_authority (CGI *cgi, char *host)
+char *cgi_cookie_authority (CGI *cgi, const char *host)
 {
   HDF *obj;
   char *domain;
@@ -1635,8 +1656,9 @@ char *cgi_cookie_authority (CGI *cgi, char *host)
  * http://www.ietf.org/rfc/rfc2109.txt
  */
 
-NEOERR *cgi_cookie_set (CGI *cgi, char *name, char *value, char *path, 
-        char *domain, char *time_str, int persistent, int secure)
+NEOERR *cgi_cookie_set (CGI *cgi, const char *name, const char *value,
+                        const char *path, const char *domain,
+                        const char *time_str, int persistent, int secure)
 {
   NEOERR *err;
   STRING str;
@@ -1675,7 +1697,7 @@ NEOERR *cgi_cookie_set (CGI *cgi, char *name, char *value, char *path,
     }
     err = string_append(&str, "\r\n");
   } while (0);
-  if (err) 
+  if (err)
   {
     string_clear(&str);
     return nerr_pass(err);
@@ -1687,7 +1709,8 @@ NEOERR *cgi_cookie_set (CGI *cgi, char *name, char *value, char *path,
 
 /* This will actually issue up to three set cookies, attempting to clear
  * the damn thing. */
-NEOERR *cgi_cookie_clear (CGI *cgi, char *name, char *domain, char *path)
+NEOERR *cgi_cookie_clear (CGI *cgi, const char *name, const char *domain,
+                          const char *path)
 {
   if (path == NULL) path = "/";
   if (domain)
